/****************************************************************************/
/*                                                                          */
/* IMG* Image Processing Tools Library                                      */
/* Program:   imgInfo.c                                                     */
/* Author:    Simon A.J. Winder                                             */
/* Date:      Thu Oct 20 20:45:45 1994                                      */
/* Copyright (C) 1994 Simon A.J. Winder                                     */
/*                                                                          */
/****************************************************************************/

#include "tools.h"

#define PRGNAME "Info"
#define ERROR(e) imgError(e,PRGNAME)

int main(void)
{
  it_image *image;
  char *type_name;
  int width,height;
  double min_value,max_value;
  
  fprintf(stderr,PRGNAME": started\n");
  
  do {
    image=i_read_image_file(stdin,IT_ANY,IM_FRAGMENT);
    if(image==NULL)
      ERROR("can't import image file");
    width=image->width;
    height=image->height;
    min_value=image->min_value;
    max_value=image->max_value;
    switch(image->type)
      {
      case IT_BIT:
	type_name="Bit image (pbm)";
	min_value=0.0;
	max_value=1.0;
	break;
      case IT_BYTE:
	type_name="Byte image (pgm)";
	min_value=0.0;
	max_value=255.0;
	break;
      case IT_RGB:
	type_name="RGB image (ppm)";
	min_value=0.0;
	max_value=255.0;
	break;
      case IT_FLOAT:
	type_name="Float image";
	break;
      case IT_DOUBLE:
	type_name="Double image";
	break;
      case IT_COMPLEX:
	type_name="Complex image";
	break;
      case IT_POLAR:
	type_name="Polar image";
	break;
      case IT_LONG:
	type_name="Long image";
	break;
      default:
	type_name="unrecognised image";
	break;
      }
    fprintf(stderr,
	    PRGNAME": type       %s.\n"
	    PRGNAME": size       (%dx%d)\n"
	    PRGNAME": valid area (%d,%d)(%dx%d)\n"
	    PRGNAME": min/max    (%g,%g)\n",
	    type_name,width,height,image->valid_x,image->valid_y,
	    image->valid_width,image->valid_height,min_value,max_value);
    i_destroy_image(image);
  } while(!feof(stdin));
  
  fprintf(stderr,PRGNAME": finished\n");
  return(0);
}
/* Version 1.0 (Oct 1994) */
