/****************************************************************************/
/*                                                                          */
/* IMG* Image Processing Tools Library                                      */
/* Program:   imgMarrHil.c                                                  */
/* Author:    Simon A.J. Winder                                             */
/* Date:      Thu Oct 20 20:45:48 1994                                      */
/* Copyright (C) 1994 Simon A.J. Winder                                     */
/*                                                                          */
/****************************************************************************/

#include "tools.h"

#define PRGNAME "MarrHil"
#define ERROR(e) imgError(e,PRGNAME)

int main(int argc,char **argv)
{
  it_image *in1,*out1=NULL;
  int width=0,height=0,x,y,dim;
  it_float *ptr1,*r0_ptr,*r1_ptr,*r2_ptr;
  int i,v_x=0,v_y=0,v_width=0,v_height=0,y_end=0;
  double val,sum,min,max,centre;

  fprintf(stderr,PRGNAME": started\n");

  dim=1;
  if(argc>2)
    ERROR("invalid arguments");
  if(argc==2)
    dim=atoi(argv[1]);
  if(dim<1)
    ERROR("bad argument values");

  /* Loop for all images */
  do {
    /* Loop for all the dimensions */
    for(i=0;i<dim;i++)
      {
	/* Load float image */
	in1=i_read_image_file(stdin,IT_FLOAT,IM_FRAGMENT);
	if(in1==NULL)
	  ERROR("can't import image file");

	if(i==0)
	  {
	    /* First time, so create a new output image */
	    width=in1->width;
	    height=in1->height;
	    v_x=in1->valid_x+1;
	    v_y=in1->valid_y+1;
	    v_width=in1->valid_width-2;
	    v_height=in1->valid_height-2;
	    y_end=v_y+v_height;
	    
	    out1=i_create_image(width,height,IT_FLOAT,IM_FRAGMENT);
	    if(out1==NULL)
	      ERROR("out of memory");

	    /* Guarentee its zero'd */
	    for(y=0;y<height;y++)
	      {
		ptr1=im_float_row(out1,y);
		for(x=0;x<width;x++)
		  *ptr1++ =0.0;
	      }
	  }
	else
	  if(in1->width!=width || in1->height!=height)
	    ERROR("images are of different sizes");

	/* Compute Laplacian multiplied by centre value and accumulate */
	for(y=v_y;y<y_end;y++)
	  {
	    r0_ptr=im_float_row(in1,y-1)+v_x;
	    r1_ptr=im_float_row(in1,y)+v_x-1;
	    r2_ptr=im_float_row(in1,y+1)+v_x;
	    ptr1=im_float_row(out1,y)+v_x;
	    for(x=0;x<v_width;x++)
	      {
		sum=*r0_ptr++;
		sum+= *r2_ptr++;
		sum+= *r1_ptr++;
		centre=(*r1_ptr++);
		sum-= (4.0*centre);
		sum+= *r1_ptr--;
		*ptr1 +=(sum*centre);
		ptr1++;
	      }
	  }
	i_destroy_image(in1);
      }
    /* Compute min and max values then export */
    imgInitMinMax(min,max);
    for(y=v_y;y<y_end;y++)
      {
	ptr1=im_float_row(out1,y)+v_x;
	for(x=0;x<width;x++)
	  {
	    val=*ptr1++;
	    if(val>max) max=val;
	    if(val<min) min=val;
	  }
      }
    out1->min_value=min;
    out1->max_value=max;
    out1->valid_x=v_x;
    out1->valid_y=v_y;
    out1->valid_width=v_width;
    out1->valid_height=v_height;

    i_write_image_file(stdout,out1,IF_BINARY);
    i_destroy_image(out1);
  } while(!feof(stdin));

  fprintf(stderr,PRGNAME": finished\n");
  return(0);
}
/* Version 1.0 (Oct 1994) */
