/****************************************************************************/
/*                                                                          */
/* IMG* Image Processing Tools Library                                      */
/* Program:   imgNCanny.c                                                   */
/* Author:    Simon A.J. Winder                                             */
/* Date:      Thu Oct 20 20:45:49 1994                                      */
/* Copyright (C) 1994 Simon A.J. Winder                                     */
/*                                                                          */
/****************************************************************************/

#include "tools.h"

#define PRGNAME "NCanny"
#define ERROR(e) imgError(e,PRGNAME)

int main(int argc,char **argv)
{
  it_image *in[10],*out1=NULL;
  int width=0,height=0,x,y,dim,i;
  int v_x=0,v_y=0,v_width=0,v_height=0,y_end=0;
  double val,min,max,ps1,ps2,ps3,ps4,ps5,ps6;
  register double temp,ur,d1d1ur,d2d2ur,d1d2ur,d1ur,d2ur;
  it_float *r0_ptr[10],*r1_ptr[10],*r2_ptr[10],*ptr1;

  fprintf(stderr,PRGNAME": started\n");

  dim=1;
  if(argc>2)
    ERROR("invalid arguments");
  if(argc==2)
    dim=atoi(argv[1]);
  if(dim<1 || dim>10)
    ERROR("bad argument values");

  /* Loop for all images */
  do {
    /* Loop for all the dimensions */
    for(i=0;i<dim;i++)
      {
	/* Load float image */
	in[i]=i_read_image_file(stdin,IT_FLOAT,IM_FRAGMENT);
	if(in[i]==NULL)
	  ERROR("can't import image file");

	if(i==0)
	  {
	    /* First time, so create a new output image */
	    width=in[i]->width;
	    height=in[i]->height;
	    v_x=in[i]->valid_x+1;
	    v_y=in[i]->valid_y+1;
	    v_width=in[i]->valid_width-2;
	    v_height=in[i]->valid_height-2;
	    y_end=v_y+v_height;
	    
	    out1=i_create_image(width,height,IT_FLOAT,IM_FRAGMENT);
	    if(out1==NULL)
	      ERROR("out of memory");
	  }
	else
	  if(in[i]->width!=width || in[i]->height!=height)
	    ERROR("images are of different sizes");
      }
    
    /* Compute 2nd derivative in direction of gradient (vector approach) */
    imgInitMinMax(min,max);
    for(y=v_y;y<y_end;y++)
      {
	/* Get a whole load of pointers: 3 rows of each component image */
	for(i=0;i<dim;i++)
	  {
	    r0_ptr[i]=im_float_row(in[i],y-1)+v_x-1;
	    r1_ptr[i]=im_float_row(in[i],y)+v_x-1;
	    r2_ptr[i]=im_float_row(in[i],y+1)+v_x-1;
	  }
	ptr1=im_float_row(out1,y)+v_x;
	for(x=0;x<v_width;x++)
	  {
	    /* Zero the partial sums */
	    ps1=ps2=ps3=ps4=ps5=ps6=0.0;
	    for(i=0;i<dim;i++)
	      {
		/* Compute all the derivative bits and accumulate */
		d1d2ur=-(*r0_ptr[i]++);
		d1d1ur=(*r1_ptr[i]++);
		d1ur=-d1d1ur;
		d1d2ur+=(*r2_ptr[i]++);
		d2d2ur=(*r0_ptr[i]++);
		d2ur=d2d2ur;
		ur=(*r1_ptr[i]++);
		temp=2.0*ur;
		d1d1ur-=temp;
		d2d2ur-=temp;
		temp=(*r2_ptr[i]++);
		d2d2ur+=temp;
		d2ur-=temp;
		d1d2ur+=(*r0_ptr[i]--);
		temp=(*r1_ptr[i]--);
		d1d1ur+=temp;
		d1ur+=temp;
		d1d2ur-=(*r2_ptr[i]--);
		d1d2ur/=4.0;
		d1ur/=2.0;
		d2ur/=2.0;
		
		/* Accumulate partial sums */
		ps1+=ur*d1d1ur;
		ps2+=ur*d2d2ur;
		ps3+=ur*d1d2ur;
		ps4+=d1ur*d1ur;
		ps5+=d2ur*d2ur;
		ps6+=d1ur*d2ur;
	      }
	    /* Compute result */
	    val=ps1*ps4+ps2*ps5+2.0*ps3*ps6;
	    *ptr1++=val;
	    if(val>max) max=val;
	    if(val<min) min=val;
	  }
      }
    /* Free all the storage */
    for(i=0;i<dim;i++)
      i_destroy_image(in[i]);

    out1->min_value=min;
    out1->max_value=max;
    out1->valid_x=v_x;
    out1->valid_y=v_y;
    out1->valid_width=v_width;
    out1->valid_height=v_height;

    i_write_image_file(stdout,out1,IF_BINARY);
    i_destroy_image(out1);
  } while(!feof(stdin));

  fprintf(stderr,PRGNAME": finished\n");
  return(0);
}
/* Version 1.0 (Oct 1994) */
